classdef PVS_checker < handle
    % class contains all the functionality specific to the PVS theorem
    % prover
    
    properties
      
        default_type = 'real';
        pvs_version = 4.2;
 
        data = [];
        mode = []; % 0 - graphical, 1 - command
    end
    
    % Static Functions
    methods(Static)
    
        %% parse_prf_file
        %   function will parse a pvs prf file inputed as a string and 
        %   return the results parsed into a stucture, parseing based on
        %   "Batch Proving and Proof Scripting in PVS" section 3.
        % inputs:
        %   string:string - string containing the read in prf file.
        % outputs;
        %   theory_id:string - Theory name
        %   decls:structure - structure of the declarations for the theory
        function [theory_id, decls] = parse_prf_file(string)
                decls = [];    
                
                if string(1) ~= '('
                   msgbox('error reading file')
                end
                i = 2;
                theory_id = [];
                % find theory name
                while (string(i) ~= ' ')
                    theory_id = [theory_id string(i)];
                    i = i + 1;
                end
                % addvance to next (
                while (string(i) ~= '(')
                    i = i+1;
                end
                
                while(string(i) ~= ')')
                    if (string(i) == '(')
                        pos = i;
                        [decl_id_temp, pos_temp] = PVS_checker.parse_decl_id(string,pos)
                        decls = [decls decl_id_temp];
                        i = pos_temp;
                    end
                    i = i + 1;
                end
        end
        
        %% parse_decl_id
        %   function will parse the theory declaration portion of the prf
        %   file
        % inputs:
        %   string:string - string containing the read in prf file.
        %   pos:int - position to start reading the string from.
        % outputs;
        %   decl_id:string - the declaration id name
        %   new_pos:int - the position where the ending char was found.
        function [decl_id,new_pos] = parse_decl_id(string,pos)
            i = pos + 1;
            decl_id.id = [];
            decl_id.default_proof_posn = [];
            decl_id.ids = [];
            
            while(string(i) ~= ' ')
               decl_id.id = [decl_id.id string(i)];
               i = i + 1;
            end
            
            while(string(i) == ' ' || string(i) == 10 || string(i) == 13)
                i = i + 1;
            end
            
            
            while(string(i) ~= ' ')
                decl_id.default_proof_posn = [decl_id.default_proof_posn string(i)];
                i = i + 1;
            end
            
            while(string(i) ~= '(')
                i = i + 1;
            end
            
            
            while(string(i) ~= ')')
                if (string(i) == '(')
                    pos = i;
                    [id_temp, pos_temp] = PVS_checker.parse_id(string,pos)
                    decl_id.ids = [decl_id.ids id_temp];
                    i = pos_temp
                end
                i = i + 1;
            end
            new_pos = i;
            

        end

        %% parse_id
        %   function will parse the individual ids of the decls
        % inputs:
        %   string:string - string containing the read in prf file.
        %   pos:int - position to start reading the string from.
        % outputs;
        %   id:structure - structure containing the parsed prf id section
        %   new_pos:int - the position where the ending char was found.
        function [id,new_pos] = parse_id(string,pos)
            i = pos + 1;
            id.id = [];
            id.description = [];
            id.run_date = [];
            id.create_date = [];
            id.status = [];
            id.refers_to = [];
            id.real_time = [];
            id.run_time = [];
            id.interactive = [];
            id.decision_proc = [];
            
            
            while(string(i) ~= ' ')
               id.id = [id.id string(i)];
               i = i + 1;
            end
            
            i = i + 1;
            
            % find the description
            if (string(i) == '"')
                stop_char = '"';
            else
                stop_char = ' ';
            end
            
            while(string(i) ~= stop_char)
               id.description = [id.description string(i)];
               i = i + 1;
            end
            
            if (string(i) == '"')
                i = i + 1;
            end
            
            % get the create_date
            i = i + 1;
            
            while(string(i) ~= ' ')
               id.create_date = [id.create_date string(i)];
               i = i + 1;
            end
            
            % get the run date
            i = i + 1;
            
            while(string(i) ~= ' ')
               id.run_date = [id.run_date string(i)];
               i = i + 1;
            end
            
            while(string(i) ~= '(')
                i = i + 1;
            end
            
            [id.script, new_pos] = PVS_checker.parse_paren(string,i);
            
            i = new_pos + 1;
            while(string(i) == ' ' || string(i) == 10 || string(i) == 13)
                i = i + 1;
            end
            
            %get the status
            
            
            while(string(i) ~= ' ')
               id.status = [id.status string(i)];
               i = i + 1;
            end
            
            while(string(i) == ' ' || string(i) == 10 || string(i) == 13)
                i = i + 1;
            end
            
            % get the refers_to
            
            if (string(i) == '(')
                [id.refers_to, new_pos] = PVS_checker.parse_paren(string,i);
                i = new_pos + 1;
            else
                while(string(i) ~= ' ')
                    id.refers_to = [id.refers_to string(i)];
                    i = i + 1;
                end
            end
            
         
            
            % get the real_time
            
            while(string(i) == ' ' || string(i) == 10 || string(i) == 13)
                i = i + 1;
            end
            
            while(string(i) ~= ' ')
               id.real_time = [id.real_time string(i)];
               i = i + 1;
            end
            
            
            % get the run_time
             while(string(i) == ' ' || string(i) == 10 || string(i) == 13)
                i = i + 1;
            end
            
            while(string(i) ~= ' ' )
               id.run_time = [id.run_time string(i)];
               i = i + 1;
            end
            % get the interactive
            
             while(string(i) == ' ' || string(i) == 10 || string(i) == 13)
                i = i + 1;
            end
            
            while(string(i) ~= ' ')
               id.interactive = [id.interactive string(i)];
               i = i + 1;
            end
            
            % get the decision_proc
             while(string(i) == ' ' || string(i) == 10 || string(i) == 13)
                i = i + 1;
            end
            
            while(string(i) ~= ')')
               id.decision_proc = [id.decision_proc string(i)];
               i = i + 1;
            end
            new_pos = i;
            
        end
        
        %% parse_paren
        %   function will output the string contained within parenthesis,
        %   including other opening and closing parenthesis.
        % inputs:
        %   string:string - string containing parenthesis
        %   pos:int - position to start reading the string from, the
        %       position of the oppening parenthesis
        % outputs;
        %   paren_string:string - the string contained within the
        %       parenthesis
        %   new_pos:int - the position where the ending parenthesis was found.
        function [paren_string, new_pos] = parse_paren(string,pos)
            % pos points to opening (
            paren_string = string(pos);
            i = pos+1;
            
            while( string(i) ~= ')')
                if (string(i) == '(')
                    [paren_string_temp, new_pos] = PVS_checker.parse_paren(string,i);
                    paren_string = [paren_string paren_string_temp];
                    i = new_pos;
                else
                    paren_string = [paren_string string(i)];
                end
                i = i + 1;
            end
            paren_string = [paren_string string(i)];
            new_pos = i;
            
        end

    end
    
    
    methods
        
        %% PVS_checker
        %   constructor
        % inputs:
        %   data - data object
        % outputs;
        %   object - created object
        function object = PVS_checker(data)
            object.data = data;
        end
        
        %% generate_pvs_file
        %   function will create a pvs file representing the table stored
        %   in data.
        % inputs:
        %   object:PVS_checker - PVS_checker object
        %   filename:string - string representing file name excluding
        %   extension
        % outputs;
        %   none
        function [] = generate_pvs_file(object,filename)
            fileid = fopen([filename '.pvs'],'w');
            code = [];
            code = sprintf('%s:THEORY\nBEGIN\n',object.data.function_name);
             
            code = [code object.pvs_check_for_imports];
            
            code = [code object.user_imports];
            % declare variables for each input
            
            %inputs = get(object.gui.function_inputs_control,'String');
           %inputs = regexp(inputs,',','split');
                       parsed_inputs = EMLGenerator.parse_inputs(object.data.function_inputs);
                       
           % initialize inputs to zero
           % functions are assumed to be total, so 0 is just for 
           % convienence
           inputs_new = [];
           for i=1:size(parsed_inputs,2)
                % set to zero
                if(size(parsed_inputs{i},2) == 2)
                    inputs_new = [inputs_new char(parsed_inputs{i}(1)) ':' char(parsed_inputs{i}(2))];
                else
                    inputs_new = [inputs_new char(parsed_inputs{i}(1)) ':' object.default_type];
                end
                
                if(i~=(size(parsed_inputs,2)))
                    inputs_new = [inputs_new ','];
                end
           end
           
           code = [code sprintf('%s(%s):real = \n',object.data.function_name,inputs_new)];
            code = [code object.generate_pvs(object.data.Grid1,object.data.Grid2,0)];
            code = [code sprintf('\nEND %s',object.data.function_name)];
            fprintf(fileid,code);

            fclose(fileid);
            
        end
        
         
        %% pvs_check
        %   function will generate a pvs file for table, and depending on
        %   user input 
        % inputs:
        %   object:PVS_checker - PVS_checker object
        % outputs;
        %   check:boolean - 0 if typecheck fails, 1 if succeds
        %   output:structure - if typecheck succeds then output contains
        %       parsed output from pvs 
        function [check,output] = pvs_check(object)
            check = 0;
            output = [];
            % file name will be expression_name.m
            object.generate_pvs_file(object.data.function_name);
            
            
            
            
            % THIS SECTION NEEDS SOME WORK
            %-------------
            % delete the existing proof
            % update (22/07/10):
            % has been fixed up now checks if prf file exists and prompts
            % user to overwrite it.
            
            exists = 0;
            button = '';
            if (exist([object.data.function_name '.prf'],'file'))
                exists = 1;
                button = questdlg(['A proof file already exists for function ' object.data.function_name sprintf('\n') ' What would you like to do' sprintf('\n') 'Note: Attempting to prove will delete existing prf file'],'prf exists','Attempt to prove','Open PVS','Cancel','Cancel') 
            end
            
            if (exists == 1 && (isempty(button) || strcmp(button,'Cancel')))
                % do nothing
            elseif (exists == 1 && (strcmp(button,'Open PVS')));
                [status, result] = system(['pvs ' object.data.function_name '.pvs']);
            elseif (exists == 0 || strcmp(button,'Attempt to prove'))
                box = waitbar(0,'Generating Proof Script') 


                    delete([object.data.function_name '.prf'])


                % generate the proof script
                object.generate_pvs_script(object.data.function_name)
                waitbar(.10,box,'Running Proof') 
                %---------
                % call pvs in batch mode with script
                % ADD check that pvs actually exists in system
                [status, result] = system(['pvs -batch -v 3 -l ' object.data.function_name '.el'])
                %objectect.msgbox_scroll(result);
                waitbar(.70,box,'Parsing Results') 
                [parsed error] = object.parse_pvs_result(result);
                
                if (error == 1)
                    err_str = ['There was an error proving tabular expression ' sprintf('\n\n') 'Error string below:' sprintf('\n\n') result];
                    GUI.msgbox_scroll(err_str);
                else

                    if (isempty(parsed))
                        check = 1;
                        output = [];
                    else
                        output = parsed;
                    end
                end
                waitbar(.100,box,'Deleting Proof Script') 

                delete([object.data.function_name '.el']);
                
                delete(box);
            end
        end
        
        
        %% user_imports
        %   returns the string importing a user selected pvs file to import
        % inputs:
        %   object:PVS_checker - PVS_checker object
        % outputs;
        %   string:string - string of pvs code to add to generated file
        function [string] = user_imports(object)
            string = [];
            if ~isempty(object.data.settings.inputs)
                files = regexp(object.data.settings.inputs,',','split');
                
                for i=1:size(files,2)
                   file_no_ext = regexp(files{i},'\w*(?=\.\w*)','match');
                   string = [string sprintf('IMPORTING %s\n', char(file_no_ext)) ];
                end
            end
        end
        
        %% parse_pvs_result
        %   parse the stuff that comes back from pvs batch mode, extract
        %   the counter examples and theorems.
        % inputs:
        %   object:PVS_checker - PVS_checker object
        %   result:string - string containing text that comes back from pvs
        % outputs:
        %   output:structure - results paresed into structure for easier
        %       manipulatings
        %   error:boolean - 0 if no error, 1 if error found during parsing.
        function [output error] = parse_pvs_result(object,result)
            found = regexp(result,'\w+(?=[\.]{4,})|(?<=[\.]{4,})\w+','match');
            if (isempty(found))
                error = 1;
                output = [];
            else
                seqs = [];
                for i = 1:2:size(found,2)-1
                    if(~strcmp(char(found(i+1)),'proved'))
                        
                        found4 = [];
                        % These parsing depend on the version of PVS used,
                        % which may be a bad idea, although pvs is not
                        
                        if object.pvs_version == 4.0
                        % find the TCC that was unprovable
                            search_str = sprintf('(?<=Proving formula %s[\\n]*)%s.*?(?=[ \\n]*Rerunning step)',char(found(i)),char(found(i)));
                        elseif object.pvs_version == 4.2
                            search_str =  sprintf('(?<=Installing rewrite rule singleton_rew\\s\\s)%s.*?(?=[ \\n]*Rerunning step)',char(found(i)));
                        end
                        found2 = regexp(result,char(search_str),'match','once');


                        % find the counter examples
                        search_str = sprintf('(?<=%s.*?substitutions:).*?(?=This will undo the proof to: \\n%s)|(?<=%s.*?)No counter-examples(?=.*?%s)',char(found(i)),char(found(i)),char(found(i)),char(found(i)));
                        found3 = regexp(result,char(search_str),'match','once');
                        if (~strcmp(found3,'No counter-examples'))
                            found4 = regexp(found3,'[^\n]*(?===>)|(?<===>)[^\n]*','match');
                        end
                        seqs = [seqs struct('id', '1234a', 'TCC', found(i), 'seq', found2, 'ce', {found4})];
                    end

                end
                output = seqs;
                error = 0;
            end
        end
        
        %% matlab_to_pvs_syntax_translation
        %translate matlab syntax to pvs syntax,
        % will be added to in the future
        % will only work for functions in the prelude
        %   
        % inputs:
        %   object:PVS_checker - PVS_checker object
        %   matlab_string:string - string to be converted from matlab
        %       syntax to pvs
        % outputs;
        %   pvs_string:string - string converted to pvs syntax.
        function pvs_string = matlab_to_pvs_syntax_translation(object,matlab_string)
            pvs_string = regexprep(matlab_string,'&&',' AND ');
            pvs_string = regexprep(pvs_string,'~(?!=)',' NOT ');
            pvs_string = regexprep(pvs_string,'~=',' /= ');
            pvs_string = regexprep(pvs_string,'==',' = ');
            pvs_string = regexprep(pvs_string,'\|\|',' OR ');
            pvs_string = regexprep(pvs_string,'ceil','ceiling');
        end
        
        
       
        
        %% genereate_pvs
        %   function will generate the pvs conditional code for a given
        %   table
        % inputs:
        %   object:PVS_checker - PVS_checker object
        %   g1:Grid - left grid
        %   g2:Grid - top grid
        %   depth:int - number representing depth inside the table
        % outputs:
        %   code:string - pvs code
        %
        function code = generate_pvs(object,g1,g2,depth)
          g1cond1 = g1.cells(1).cond_text;
          g2cond1 = g2.cells(1).cond_text;
          code = [];
          %1D horizontal
          if(~isempty(g1cond1) && isempty(g2cond1))
            code = [code object.generate_pvs_horizontal(g1,g2.cells(1),depth)];
          % something in vertical column
          else

            code = [code sprintf('COND\n')];
            elsecell = [];
            for i=1:size(g2.cells,2)
                g2cond = g2.cells(i).cond_text;
                % condition string of otherwise corresponds to an else
                % statement, we allow this statement to be in any order
                % in the cells of the grid, so we need to find where it
                % is, if it exists.
                if (strcmp(g2cond,'otherwise'))
                    elsecell = g2.cells(i);
                    found = 1;
                    continue
                end
                code = [code sprintf('%s->',object.matlab_to_pvs_syntax_translation(strtrim(char(g2.cells(i).cond_text))))];
                
                if(~isempty(g2.cells(i).subgrid))
                    code = [code object.generate_pvs(g1,g2.cells(i).subgrid,depth)];
                else
                    if (~isempty(g1cond1))
                         code = [code object.generate_pvs_horizontal(g1,g2.cells(i),depth)];
                    else    
                
                        resultcell = object.data.Grid0.search_return(g1.cells(1),g2.cells(i));
                        if(~isempty(resultcell))
                            code = [code object.matlab_to_pvs_syntax_translation(strtrim(char(resultcell.result_text)))];

                            
                        else
                        end
                    end
                end
                if(isempty(elsecell))
                    if(i~= size(g2.cells,2))
                        code = [code sprintf(',\n')];
                    else
                        code = [code sprintf('\n')];
                    end
                else
                    code = [code sprintf(',\n')];
                end      
              
            end
           if(~isempty(elsecell))
                 %resultcell = object.Grid0.search_return(elsecell,elsecell);
                code = [code sprintf('ELSE->')];

                if(~isempty(elsecell.subgrid))
                    code = [code object.generate_pvs(g1,elsecell.subgrid,depth) sprintf('\n')];
                else
                    if (~isempty(g1cond1))
                         code = [code object.generate_pvs_horizontal(g1,elsecell,depth) sprintf('\n')];
                    else    
                
                        resultcell = object.data.Grid0.search_return(g1.cells(1),elsecell);
                        if(~isempty(resultcell))
                            code = [code object.matlab_to_pvs_syntax_translation(strtrim(char(resultcell.result_text))) sprintf('\n')];

                        end
                    end
                end
                
           end

         code = [code sprintf('ENDCOND')];
 
          end
        end
        
        
        %% generate_pvs_horizontal
        %   generate the pvs code for a condition in the horizontal(top)
        %   grid, done in seperate function because it will get called
        %   multiple times for the same condition in a 2d table.
        % inputs:
        %   object:PVS_checker - PVS_checker object
        %   g1:Grid - left grid
        %   g2:Grid - top grid
        %   depth:int - number representing depth inside the table
        % outputs:
        %   code:string - pvs code
        %
        function code = generate_pvs_horizontal(object,g1,g2_cell,depth)

        code = [];

            code = [code sprintf('COND\n')];
            elsecell = [];
            for i=1:size(g1.cells,2)
                g1cond = g1.cells(i).cond_text;
                % condition string of otherwise corresponds to an else
                % statement, we allow this statement to be in any order
                % in the cells of the grid, so we need to find where it
                % is, if it exists.
                if (strcmp(g1cond,'otherwise'))
                    elsecell = g1.cells(i);
                    found = 1;
                    continue
                end
                code = [code sprintf('%s->',object.matlab_to_pvs_syntax_translation(strtrim(char(g1.cells(i).cond_text))))];
                resultcell = object.data.Grid0.search_return(g1.cells(i),g2_cell);
                if(~isempty(resultcell))
                    code = [code object.matlab_to_pvs_syntax_translation(strtrim(char(resultcell.result_text)))];
                    
                    if(isempty(elsecell))
                        if(i~= size(g1.cells,2))
                            code = [code sprintf(',\n')];
                        else
                            code = [code sprintf('\n')];
                        end
                    else
                        code = [code sprintf(',\n')];
                    end
                else
                end
            end
            if(~isempty(elsecell))
                 resultcell = object.data.Grid0.search_return(elsecell,g2_cell);
                code = [code sprintf('ELSE->%s\n',object.matlab_to_pvs_syntax_translation(strtrim(char(resultcell.result_text))))];

            end
            code = [code sprintf('ENDCOND')];

       
            
     
        end
        
        %% pvs_check_for_imports
        %   checks both horizontal and vertical grids for functions for
        %   which there is a file on the path of the same name, in which
        %   case create pvs code to import the pvs file for that function
        % inputs:
        %   object:PVS_checker - PVS_checker object
        % outputs:
        %   string:string - pvs code of imports
        %
        function string = pvs_check_for_imports(object)
            [string,found] = object.pvs_check_for_imports_g(object.data.Grid2,{});
            [string2,found2] = object.pvs_check_for_imports_g(object.data.Grid1,found);
            string = [string string2];
        end
        
        %% pvs_check_for_imports_g
        %   recursively check a grid for an imported functions, see comment
        %   for pvs_check_for_imports
        % inputs:
        %   object:PVS_checker - PVS_checker object
        %   grid:Grid - grid to check
        %   found:array - list of functions already imported so we dont
        %       import the same thing twice.
        % outputs:
        %   string:string - pvs code
        %   found:array - updated list of found functions to import
        %
        function [string,found] = pvs_check_for_imports_g(object,grid,found)
            string = [];
            for i=1:size(grid.cells,2)
                % recurse through
                if (~isempty(grid.cells(i).subgrid))
                    string = [string object.pvs_check_for_imports_g(grid.cells(i).subgrid)];
                end
                text = char(grid.cells(i).cond_text);
                
                vars = regexp(text,'([a-zA-Z][a-zA-Z0-9_]*)','match');
                
                for j=1:size(vars,2)
                   if(exist(vars{j},'file')==2 && ~strcmp(vars{j},'otherwise'))
                       %check if already imported file
                       if (~any(ismember(found,vars{j})))
                            string = [string 'IMPORTING ' vars{j} sprintf('\n')];
                            found = [found vars{j}];
                       end
                   end
                end
            end
        end
        
        
        
        %% generate_pvs_script
        %  generate the pvs script used to prove and look for counter
        %  examples for a theory. script will have name filename.el
        % inputs:
        %   object:PVS_checker - PVS_checker object
        %   filename:string - filename to save the script as, exclude
        %       extension.
        % outputs:
        %    none
        %
        function [] = generate_pvs_script(object,filename)
            fileid = fopen([filename '.el'],'w');
            code = ['(prove-tccs-pvs-file "' filename '" "(try (try (tcc) (try (assert) (skip) (fail) ) (fail))(skip) (random-test :count ' sprintf('%ld',object.data.settings.count) ' :size ' sprintf('%d',object.data.settings.range) '))"  )' sprintf('\n')];

            fprintf(fileid,code);
            fclose(fileid);
        end
        
        %% generate_pvs_status_script
        %   generate the script used for checking the status of a pvs file
        % inputs:
        %   object:PVS_checker - PVS_checker object
        %   filename:string - filename of the pvs file to checkstatus of,
        %   excluding extension
        % outputs:
        %   script_name:string - filename of the script of format name.el
        function [script_name] = generate_pvs_status_script(object,filename)
            script_name = [filename '_status' '.el'];
            fileid = fopen(script_name,'w');
            code = ['(status-proof-pvs-file "' filename '")'];

            fprintf(fileid,code);
            fclose(fileid);
        end
        
        %% check_status
        %   checks the status of a pvs_checker object by generating a
        %   status script running it a checking the results to ensure that
        %   all the theories are proved.
        % inputs:
        %   object:PVS_checker - PVS_checker object
        % outputs:
        %   status:boolean - 0 if pvs file has been checked, 1 if it has
        %   not
        function status = check_status(object)
            script_name = object.generate_pvs_status_script(object.data.function_name);
            [status, result] = system(['pvs -batch -v 3 -l ' script_name]);
            parsed = object.parse_status(result);
            pass = 1;
            if (size(parsed,2) == 0)
                pass = 0;
            end
            for i=1:2:size(parsed,2)-1
                if ~strncmp(parsed(i+1),'proved',6)
                    pass = 0;
                end
            end
            
            status = pass;
            delete(script_name);
            
        end
        
        %% parse_status
        %   parse the result of running the pvs status script. look for a
        %   list of theorems and associated status return that in parsed
        %   list.
        % inputs:
        %   object:PVS_checker - PVS_checker object
        %   string:string - string containing output of runnign pvs script
        % outputs:
        %   parsed:array - array containing each of the theorems of the pvs
        %   file followed by its status so will look something like 
        %   ['TCC1' 'proved' 'TCC2' 'unfinished']... etc
        function [parsed] = parse_status(object,string)
              thrms_status = regexp(string,'\w+(?=[\.]{4,})|(?<=[\.]{4,})\w+','match');
              parsed = thrms_status;
                
        end
        
    end
    
end

